#!/usr/bin/perl -w

package GRUBConf;

use Data::Dumper;
use Exporter;
use Gtk2;
use Glib ':constants';
use Gtk2::GladeXML;

@ISA=qw/Exporter Gtk2::Widget/;

sub new{
	my $self={};
	my $glade=Gtk2::GladeXML->new("glade/grubconf.glade",'plugin');
	$self=$glade->get_widget('plugin');
	$self->{'glade'}=\$glade;
	$glade->signal_autoconnect_from_package();
	bless $self;
	
	buildGUI($self,$glade);
	check_for_file($self);
	$self->show();
	return $self;
}

#checks for grub menu file
sub check_for_file{
	my ($self)=@_;
	
	#check for 2 files, menu.1st and grub.conf
	$self->{'file'}='/boot/grub/menu.1st';
	unless (-e $self->{'file'}){
		$self->{'file'}='/boot/grub/grub.conf';
	}
	
	#if either don't exist, warn...
	unless (-e $self->{'file'}){
		warn "No GRUB configuration file detected.\nPlease select one with \'Open\'";
	}else{
		Open($self,$self->{'file'});
	}
	

}

#just sets up references to all UI elements
#	used for referencing UI elements;
#	for better readability
sub buildGUI{
	my ($self,$glade)=@_;
	
	$self->{'has_boot_menu'}=$glade->get_widget('has_boot_menu');
		$self->{'has_boot_menu'}->set_active(1);
		
	$self->{'has_timeout'}=$glade->get_widget('has_timeout');
		$self->{'timeout_spin'}=$glade->get_widget('timeout_spin');
		$self->{'has_timeout'}->signal_connect(
			toggled=>sub{$self->sync_active($glade,'has_timeout','timeout_spin');});

	$self->{'has_default_boot'}=$glade->get_widget('has_default_boot');
		$self->{'default_root'}=$glade->get_widget('boot_default_combo');
		$self->{'default_model'}=$self->{'default_root'}->get_model();
		$self->{'has_default_boot'}->signal_connect(
			toggled=>sub{$self->sync_active($glade,'has_default_boot','boot_default_combo');});

	$self->{'has_fallback_boot'}=$glade->get_widget('has_fallback_boot');
		$self->{'fallback_root'}=$glade->get_widget('boot_fallback_combo');
		$self->{'fallback_model'}=$self->{'fallback_root'}->get_model();
			$self->{'fallback_model'}->clear;
		$self->{'has_fallback_boot'}->signal_connect(
			toggled=>sub{$self->sync_active($glade,'has_fallback_boot','boot_fallback_combo');});
	
	$self->{'has_default_color'}=$glade->get_widget('has_default_color');
		$self->{'default_fgcolor'}=$glade->get_widget('default_fgcolor')->get_child();
		$self->{'default_bgcolor'}=$glade->get_widget('default_bgcolor')->get_child();

	$self->{'has_default_hilite_color'}=$glade->get_widget('has_default_hilite_color');
		$self->{'default_hilite_fgcolor'}=$glade->get_widget('default_hilite_fgcolor')->get_child();
		$self->{'default_hilite_bgcolor'}=$glade->get_widget('default_hilite_bgcolor')->get_child();
		$self->{'has_default_hilite_color'}->signal_connect(
			toggled=>sub{
				$self->sync_active($glade,'has_default_hilite_color','default_hilite_fgcolor','default_hilite_bgcolor');
			});

		$self->{'has_default_color'}->signal_connect(
			toggled=>sub{
				$self->sync_active($glade,'has_default_color','default_fgcolor','default_bgcolor','has_default_hilite_color');
					$self->sync_active($glade,'has_default_hilite_color','default_hilite_fgcolor','default_hilite_bgcolor');
					$self->sync_active($glade,'has_default_color','default_hilite_fgcolor','default_hilite_bgcolor') if $self->{'has_default_hilite_color'}->get_active();
			});

	$self->{'has_bgimage'}=$glade->get_widget('has_bgimage');
		$self->{'bgimage_entry'}=$glade->get_widget('bgimage_entry');
		$glade->get_widget('locate_bgimage')->signal_connect(clicked=>sub{
			$glade->get_widget('bgimage_entry')->set_text(puniconf::getFileOpen("Select a compressed image..."));});
		$self->{'bgimage_root_entry'}=$glade->get_widget('bgimage_root_entry');
		$self->setDevicesModel($self->{'bgimage_root_entry'}->get_model());
		$self->{'bgimage_root_entry'}->signal_connect('changed'=>sub{$self->on_devices_selected(@_)});
		$self->{'has_bgimage'}->signal_connect(
			toggled=>sub{$self->sync_active($glade,'has_bgimage','bgimage_root_entry','bgimage_entry','locate_bgimage');});
	

	#treeview to display all menu entries
	$self->{'mView'}=$glade->get_widget('menuList_view');
		$self->{'mList'}=Gtk2::ListStore->new ('Glib::String');
		$self->{'mView'}->set_model($self->{'mList'});
		$self->{'mView'}->get_selection->set_mode ('single');
		$self->{'mView'}->append_column (
					Gtk2::TreeViewColumn->new_with_attributes ("Menu Entries",
						Gtk2::CellRendererText->new() ,text=>0));
		$self->{'mView'}->signal_connect (
					row_activated =>sub{
							on_menuList_view_row_activated( $self,@_,$self->{'glade'})
							});
	$glade->get_widget('add_entry_btn')->signal_connect('clicked'=>sub{$self->display_edit_menu_win});
	$glade->get_widget('remove_entry_btn')->signal_connect('clicked'=>sub{
			my ($model,$iter)=$self->{'mView'}->get_selection()->get_selected();
			$model->remove($iter);
			});
	$glade->get_widget('properties_btn')->signal_connect('clicked'=>sub{
				on_menuList_view_row_activated( 
						$self,
						$self->{'mView'},
						$self->{'mList'}->get_path(($self->{'mView'}->get_selection->get_selected)[1]),
						undef,
						$self->{'glade'})
			});

	$self->{'textBuffer'}=new Gtk2::TextBuffer();
		$glade->get_widget('text_view')->set_buffer($self->{'textBuffer'});


	$self->show_all;
}

sub sync_active{
	my ($self,$glade,$togglebtn,@toggled)=@_;
	foreach $w (@toggled){
		$glade->get_widget($w)->set_sensitive($glade->get_widget($togglebtn)->get_active());
	}
}

###########  Core Parsing Routines   ####################

sub parseConfig{
	my ($self)=@_;
	
	#hash containing main grub sexn
	my %main;
	$self->{'mainsection'}=\%main;
	my @entries;	#list of hashrefs that correspond to each menu
	$self->{'entries'}=\@entries;
	
	#list of each line in conf file, chomp()'ed
	my @lines=split("\n",$self->{'textBuffer'}->get_text(
										$self->{'textBuffer'}->get_start_iter,
										$self->{'textBuffer'}->get_end_iter,1
										) 
							);	
	my $inEntry=0;	#flag to tell when main section ends
	@lines=splice(@lines,$self->getMainSection(\@lines));
	$self->getEntries(@lines);

	$self->update_all_menus;
	
	$self->{'default_root'}->set_active($self->{'main'}->{'default'} || 0);
	$self->{'fallback_root'}->set_active($self->{'main'}->{'fallback'} || 0);
} #/parseConfig()


#updates all menus and lists that depend on the entry titles
sub update_all_menus{
	my ($self)=@_;
	my $iter;
	
	$self->{'default_model'}->clear;
	$self->{'mList'}->clear;
	foreach $i (@{$self->{'entries'}}){
		$iter=$self->{'default_model'}->append;
		$self->{'default_model'}->set($iter,0=>$i->{'title'});
		$iter=$self->{'fallback_model'}->append;
		$self->{'fallback_model'}->set($iter,0=>$i->{'title'});
		$iter=$self->{'mList'}->append;
		$self->{'mList'}->set($iter,0=>$i->{'title'});
	}
	
}


#get main section
sub getMainSection{
	my ($self,$lines)=@_;
	my $i=-1;
	foreach $l (@$lines){
		$i++;
		#when a 'title' is encountered, return from method
		if ($l=~/^\s*title\s/){
			return $i;
		}
		#skip line if comment
		elsif ($l=~/^\s*\#/){
			#next;
		}
		#set hiddenmenu flag to true
		elsif ($l=~/^\s*hiddenmenu\s/){
			$self->{'main'}->{"hiddenmenu"}=1;
			$self->{'has_boot_menu'}->set_active(0);
		}
#		#options with an '=' sign
#		elsif ($l=~/^\s*(\S*)\s*\=\s*(\S*)/){
#			$self->{'main'}->{"$1"}=$2;
#		}
		#options without an '=' sign
		elsif ($l=~/^\s*(\S*)\s*(\S.*)/){
			$self->{'main'}->{"$1"}=$2;
			if (lc($1) eq 'color'){
				$self->{'has_default_color'}->set_active(1);
				my $tmp=$2;
				$tmp=~/(\S*)\/?(\S*)\s+((\S*)\/?(\S*))?/;
				$self->{'default_fgcolor'}->set_text($1 || '');
				$self->{'default_bgcolor'}->set_text($2 || '');
				$self->{'has_default_hilite_color'}->set_active(1) if $4;
				$self->{'default_hilite_fgcolor'}->set_text($4 || '');
				$self->{'default_hilite_bgcolor'}->set_text($5 || '');
			}
			#default
			if (lc($1) eq 'default'){
				$self->{'has_default_boot'}->set_active(1);
			}
			#fallback
			if (lc($1) eq 'fallback'){
				$self->{'has_fallback_boot'}->set_active(1);
			}
			#timeout
			if (lc($1) eq 'timeout'){
				$self->{'has_timeout'}->set_active(1);
				$self->{'timeout_spin'}->set_value($2);
			}
			#splashimage
			if (lc($1) eq 'splashimage'){
				$self->{'has_bgimage'}->set_active(1);
				$2=~/^\s*(\(.*\))(.*)/;
				$self->{'bgimage_root_entry'}->get_child()->set_text($1);
				$self->{'bgimage_entry'}->set_text($2);
			}
		}
	}
}


sub getEntries{
	my ($self,@lines)=@_;
	my @entrynos; #list of lines that begin menu entries
	my $i=0;
	my $first=1; #is it the first entry?
	
	#loop over remaining lines and set length of entries
	foreach $l (@lines){
		if ($l=~/^\s*title\s/){
			push ( @entrynos,$i ) unless $first;
			$first=0;
			$i=0;
		}
		$i++;
	}
	
	#read through all entry lengths and put each entry hash into $self->{"entries"}
	foreach $length (@entrynos){
		push @{$self->{'entries'}}, parseEntry( splice(@lines,0,$length) );
	}
	push @{$self->{'entries'}}, parseEntry( @lines );
	
} #/getEntries()

sub parseEntry{
	my (@lines)=@_;
	my %entry;
	my @options;

	foreach $l (@lines){
		#find title
		if ($l=~/^\s*title\s+(\S.*)/){
			$entry{"title"}="$1";
		#find options with values
		}elsif ($l=~/^\s*(\S+)\s+(\S.*)/){
			$entry{"$1"}="$2";
		#find options with no values
		}elsif($l=~/^\s*(\S+)\s*/){
			push @options,$1;
		}
	}	
	$entry{"options"}=\@options;

	return \%entry;
}


#writes to main text view buffer in order of appearance in the menu view list
sub enact_changes{
	my ($self)=@_;

	$self->{'textBuffer'}->delete($self->{'textBuffer'}->get_start_iter,$self->{'textBuffer'}->get_end_iter);
	
	#warn Dumper $self;
	my $out= "#menu.1st\n#generated by ./puniconf::GRUBConf\n";
	
	unless ($self->{'has_boot_menu'}->get_active){$out.="hiddenmenu\n";}
	if ($self->{'has_timeout'}->get_active){$out.="timeout ".$self->{'timeout_spin'}->get_value_as_int."\n";}
	if ($self->{'has_default_boot'}->get_active){$out.="default ".$self->{'default_root'}->get_active."\n";}
	if ($self->{'has_fallback_boot'}->get_active){$out.="fallback ".$self->{'fallback_root'}->get_active."\n";}
	if ($self->{'has_default_color'}->get_active){
		my ($fgcolor,$bgcolor,$fghilite,$bghilite)=(
			$self->{'default_fgcolor'}->get_text,
			$self->{'default_bgcolor'}->get_text,
			$self->{'default_hilite_fgcolor'}->get_text,
			$self->{'default_hilite_bgcolor'}->get_text,
		);
		$out.="color ".$fgcolor if $fgcolor;
		$out.='/'.$bgcolor if ($fgcolor && $bgcolor);
		if ($self->{'has_default_hilite_color'}->get_active){
			$out.=" ".$fghilite if $fghilite;
			$out.='/'.$bghilite if ($fghilite && $bghilite);
		}	
		$out.="\n";
	}
	if ($self->{'has_bgimage'}->get_active && $self->{'bgimage_entry'}->get_text){
		$out.="splashimage ".$self->{'bgimage_root_entry'}->get_child->get_text.
		$self->{'bgimage_entry'}->get_text."\n";
	}
	
	
	$self->{'textBuffer'}->set_text($out);
	
	$self->{'mList'}->foreach(\&add_text_entry,$self);
	
}

#called by foreach func for recursing main menu list
#adds entries to text view
sub add_text_entry{
	#warn Dumper @_;
	my ($model,$path,$iter,$self)=@_;
	my $out="\n";
	my %tmp=%{$self->getEntryHash($model->get($iter,0))};

	if (%tmp){
		my @options=@{$tmp{'options'}} if defined($tmp{'options'});
		delete($tmp{'options'}); 
		$out.="title ".$tmp{'title'}."\n";
		delete($tmp{'title'});
		foreach $key (keys(%tmp)){
			$out.="\t$key ".$tmp{$key}."\n";
		}
		foreach $o (@options){
			$out.="\t$o\n" if $o;
		}
	}
		
	$self->{'textBuffer'}->insert($self->{'textBuffer'}->get_end_iter,$out);

}
sub getEntryHash{
	my ($self,$title)=@_;
	
	foreach $entry ( @{ $self->{'entries'} } ){
		if ($entry->{'title'} eq $title){
			return $entry;
		}	
	}
}

###########  /Core Parsing Routines   ####################

#############################################
###########  Callbacks   ####################

sub on_locate_btn_clicked{
	my ($self,$entry,$txt)=@_;
		
	$entry->set_text(puniconf::getFileOpen($txt));
}

sub on_devices_selected{
	my ($self,$combo)=@_;
	#warn Dumper @_;
	my $iter=$combo->get_active_iter();
	my $model=$combo->get_model();
	if ($iter){
		my $selected=$model->get($iter,0);
		my $corrected=$self->dev2grub($selected);
		$combo->get_child()->set_text($corrected);
	}
}

sub on_menuList_view_row_activated{
	#warn Dumper @_;
	my ($self,$view,$path,$column,$glade)=@_;
	my $iter = $view->get_model()->get_iter ($path);
	my $title=$view->get_model()->get($view->get_model()->get_iter ($path),0);
	
	#warn puniconf::getFileOpen($title);
	$self->display_edit_menu_win($title);	
	
}


###########  /Callbacks   ####################

####  Plugin Implemented Methods   ##########

sub getLabel{
	return puniconf::stockTab("GRUBConf",'gtk-harddisk','Configures the Grand Unified Bootloader');
}

sub showHelp{
	#puniconf::showStatus("This is where the help goes.");
}

sub Open{
	my ($self,$file)=@_;
	#warn Dumper @_;
	unless ($file){
		my $f=puniconf::getFileOpen("Select GRUB configuration file...");

		$self->{'file'}=$f if $f;
		puniconf::showStatus("$f was selected.");
	}
	if (-e $self->{'file'}){
		if (open(F, $self->{'file'})){
			$self->{'textBuffer'}->delete(
								$self->{'textBuffer'}->get_start_iter,
								$self->{'textBuffer'}->get_end_iter);
			foreach $l (<F>){
				$self->{'textBuffer'}->insert($self->{'textBuffer'}->get_end_iter,$l);
			}
			
			puniconf::showStatus($self->{'file'}." was opened.");

			close F;
			#create backup file
			if (open(F, $self->{'file'})){
				if (open(BAK, '>'.$self->{'file'}.".bak")){
					foreach $l (<F>){
						print BAK $l;
					}
					puniconf::err($self->{'file'}.".bak was created!");
					close F;
					close BAK;
				}else{
					warn "Cannot create backup file ".$self->{'file'}."bak!";
				}
			}
		}else{
			$self->{'textBuffer'}->insert($self->{'textBuffer'}->get_end_iter,"Can\'t open $file!");
		}
	}

	#create backup file
	if (open(F, $self->{'file'})){
		if (open(BAK, '>'.$self->{'file'}.".bak")){
			foreach $l (<F>){
				print BAK $l;
			}
			puniconf::err($self->{'file'}.".bak was created!");
			close F;
			close BAK;
		}else{
			warn "Cannot create backup file ".$self->{'file'}."bak!";
		}
	}
	
	
	
	$self->parseConfig();
}#/open()

sub Save{
	my ($self)=@_;
	if ( open(FILE, '>'.$self->{'file'} ) ){
		puniconf::showStatus("Opened file for writing: ".$self->{'file'});
		#puniconf::err($self->{'textBuffer'}->get_text($self->{'textBuffer'}->get_start_iter,$self->{'textBuffer'}->get_end_iter,1));
		#puniconf::showErr();
		print FILE $self->{'textBuffer'}->get_text($self->{'textBuffer'}->get_start_iter,$self->{'textBuffer'}->get_end_iter,1);
		close FILE;
	}
	#puniconf::getFileSave("Save GRUB configuration file...");
}

sub Focus{
	puniconf::showStatus("Configure The Grand Unified Bootloader");
}

sub getIcon{
	my $icon=puniconf::getWindow->render_icon("gtk-harddisk", "menu");
	return $icon;
}


sub getHelp{
	my ($self,$buffer)=@_;
	my $iter = $buffer->get_end_iter;

	$buffer->insert_with_tags_by_name ($iter, "./puniconf::GRUBConf\n", "helpheading");
	$buffer->insert_with_tags_by_name ($iter, "About\n", "helptopic");
	$buffer->insert_with_tags_by_name ($iter, "GRUBConf is a plugin for ./puniconf that configures ".
			"the Grand Unified Bootloader or GRUB.\n\n", "helptopicbody");
	$buffer->insert_with_tags_by_name ($iter, "Usage\n", "helptopic");
	$buffer->insert_with_tags_by_name ($iter, "Most items are fairly self-explanatory. ".
			"Almost every item has a tooltip available if you need help on it.\n".
			"There are two main tabs: GUI and Text. GUI allows you to edit the grub.conf file in a graphic interface.\n".
			"The Text tab allows you to manually edit the grub.conf file if you want something more than the GUI gives.\n".
			"The toolbar button 'Enact Changes' simply changes the Text view to reflect ".
			"the changes made from the GUI.\n".
			"When you save the file, the text in the Text view is saved to disk.".
			" This way, you can preview the file before saving to disk.", "helptopicbody");

}

sub setMenu{}

sub doAbout{
	my $glade=Gtk2::GladeXML->new("glade/grubconf.glade",'about_win');
	my $about_win=$glade->get_widget('about_win');
	$about_win->add_events ([qw(key-press-mask button-press-mask)]);
	$about_win->signal_connect(key_press_event=>sub{$about_win->destroy;1;});
	$about_win->signal_connect(button_press_event=>sub{$about_win->destroy;1;});
	puniconf::err($about_win->get_events);
	$about_win->show_all;

}

sub doHelp{}

sub getToolbarItems{
	my ($self)=@_;
	my $newBtnToolTip="Update All Changes\n(do not save)";
	my $icon=Gtk2::Image->new_from_stock('gtk-execute','large-toolbar');
	warn 'Image cannot be created!' unless $icon;
	my $newBtn=Gtk2::ToolButton->new ($icon,'Enact Changes');
		$newBtn->signal_connect (clicked => sub{enact_changes($self)});
	puniconf::addTbItem($newBtn,$newBtnToolTip,$newBtnToolTip);
}

####  / Plugin Implemented Methods   ##########


###########   Edit Menu Window   ##############

sub display_edit_menu_win{
	my ($self,$title)=@_;
	my $eref;
	my %tmp;

	#get glade info for the menu editing window
	my $glade=Gtk2::GladeXML->new("glade/grubconf.glade",'edit_menu_win');
	$self->{'edit_menu_win'}=$glade->get_widget('edit_menu_win');
	
	#setup cancel button
	$glade->get_widget('edit_menu_win_cancel')->signal_connect (
				clicked=>sub{$self->{'edit_menu_win'}->destroy}
				);
	#setup ok button
	$glade->get_widget('edit_menu_win_ok')->signal_connect (
				clicked =>sub{on_menu_edit_win_ok_clicked($self,$glade,$eref)}
				);	

	$self->{'title_entry'}=$glade->get_widget('title_entry');

	$glade->get_widget('has_chainloader')->signal_connect(
			toggled=>sub{$self->sync_active($glade,'has_chainloader','chainloader_entry');});
	$self->{'chainloader_entry'}=$glade->get_widget('chainloader_entry');

	$glade->get_widget('has_initrd')->signal_connect(
			toggled=>sub{$self->sync_active($glade,'has_initrd','initrd_entry','initrd_locate_btn');});
	$glade->get_widget('initrd_locate_btn')->signal_connect(clicked=>sub{
			$glade->get_widget('initrd_entry')->set_text(puniconf::getFileOpen("Select a ramdisk image..."));});
	$self->{'initrd_entry'}=$glade->get_widget('initrd_entry');

	$glade->get_widget('has_kernel')->signal_connect(
			toggled=>sub{$self->sync_active($glade,'has_kernel','kernel_entry','kernel_locate_btn','kernel_param_entry');});
	$glade->get_widget('kernel_locate_btn')->signal_connect(clicked=>sub{
			$glade->get_widget('kernel_entry')->set_text(puniconf::getFileOpen("Select a kernel image..."));});
	$self->{'kernel_entry'}=$glade->get_widget('kernel_entry');
	$self->{'kernel_param_entry'}=$glade->get_widget('kernel_param_entry');

	$self->{'root_location_menu'}=$glade->get_widget('root_location_combo');
	$self->setDevicesModel($self->{'root_location_menu'}->get_model());
	$self->{'root_location_menu'}->signal_connect('changed'=>sub{$self->on_devices_selected(@_)});

	$self->{'root_location_entry'}=$glade->get_widget('root_location_combo')->get_child();
	$self->{'root_type_menu'}=$glade->get_widget('root_type_menu');

	$glade->get_widget('has_hilite_color')->signal_connect(
		toggled=>sub{$self->sync_active($glade,'has_hilite_color','color_hilite_fgcolor','color_hilite_bgcolor');
		});
	$self->{'color_fgcolor'}=$glade->get_widget('color_fgcolor')->get_child();
	$self->{'color_bgcolor'}=$glade->get_widget('color_bgcolor')->get_child();
	$self->{'color_hilite_fgcolor'}=$glade->get_widget('color_hilite_fgcolor')->get_child();
	$self->{'color_hilite_bgcolor'}=$glade->get_widget('color_hilite_bgcolor')->get_child();

		$glade->get_widget('has_normal_color')->signal_connect(
		toggled=>sub{
			$self->sync_active($glade,'has_normal_color','color_fgcolor','color_bgcolor','has_hilite_color');
				$self->sync_active($glade,'has_hilite_color','color_hilite_fgcolor','color_hilite_bgcolor');
				$self->sync_active($glade,'has_normal_color','color_hilite_fgcolor','color_hilite_bgcolor') if $glade->get_widget('has_hilite_color')->get_active();
		});

		$self->{'extra_options_view'}=$glade->get_widget('extra_options_view');
	$self->{'extra_options_combo'}=$glade->get_widget('add_options_combo');
	$glade->get_widget('add_options_btn')->signal_connect('clicked'=>sub{
			my $iter=$self->{'extra_options_view'}->get_model->append();
			$self->{'extra_options_view'}->get_model->set($iter,0,$self->{'extra_options_combo'}->get_child->get_text);
	});
	
	$glade->get_widget('delete_options_btn')->signal_connect('clicked'=>sub{
		my ($model,$iter)=$self->{'extra_options_view'}->get_selection()->get_selected();
		$model->remove($iter);
	});

	$self->{'options_list'}=Gtk2::ListStore->new ('Glib::String','Glib::String');
	$self->{'extra_options_view'}->set_model($self->{'options_list'});
	$self->{'extra_options_view'}->get_selection->set_mode ('single');
	$self->{'extra_options_view'}->append_column (
				Gtk2::TreeViewColumn->new_with_attributes ("Option",
					Gtk2::CellRendererText->new() ,text=>0));
	$self->{'extra_options_view'}->append_column (
				Gtk2::TreeViewColumn->new_with_attributes ("Value",
					Gtk2::CellRendererText->new() ,text=>1));
	$self->{'extra_options_view'}->signal_connect (
				row_activated =>sub{
						$self->on_edit_menu_options_row_activated($self->{'options_list'});
						});
	
	if ($title){
		#get selected entry hash reference
		$eref=$self->getEntryHash($title);
		#copy hash to a temp hash, so options can be eliminated
		%tmp=%{$eref};
					
		#setup other relevant UI elements
		$self->{'title_entry'}->set_text(($tmp{'title'} || ''));
		delete ($tmp{'title'});
		
		$glade->get_widget('has_chainloader')->set_active(1) if ($tmp{'chainloader'});
		$self->{'chainloader_entry'}->set_text(($tmp{'chainloader'} || ''));
		delete ($tmp{'chainloader'});
	
		$glade->get_widget('has_initrd')->set_active(1) if ($tmp{'initrd'});
		$self->{'initrd_entry'}->set_text(($tmp{'initrd'} || ''));
		delete ($tmp{'initrd'});
	
		#split kernel from parameters
		$glade->get_widget('has_kernel')->set_active(1) if ($tmp{'kernel'});
		if ($tmp{'kernel'}){
			$tmp{'kernel'}=~/^(\S*)\s(\S.*)/;
			$self->{'kernel_entry'}->set_text(($1 || ''));
			$self->{'kernel_param_entry'}->set_text(($2 || ''));
		}
		delete ($tmp{'kernel'});
		
		$self->{'root_location_entry'}->set_text(($tmp{'root'} || $tmp{'rootnoverify'} || ''));
		$self->{'root_type_menu'}->set_active(1) if $tmp{'rootnoverify'};
		$self->{'root_type_menu'}->set_active(0) if $tmp{'root'};
		delete ($tmp{'rootnoverify'}) if $tmp{'rootnoverify'};
		delete ($tmp{'root'}) if $eref->{'root'};
		
		if ($tmp{'color'}){
			$tmp{'color'}=~/(\S*)\/?(\S*)\s+((\S*)\/?(\S*))?/;
			$self->{'color_fgcolor'}->set_text($1 || '');
			$self->{'color_bgcolor'}->set_text($2 || '');
			$self->{'color_hilite_fgcolor'}->set_text($4 || '');
			$self->{'color_hilite_bgcolor'}->set_text($5 || '');
		}		
		my $iter; #create before loops, for speed
		#display the rest of entry
		foreach $o ( @{$tmp{'options'}} ){
			$iter=$self->{'options_list'}->append;
			$self->{'options_list'}->set($iter,0,$o);
		}
		delete($tmp{'options'});
		
		foreach $o ( keys(%tmp) ){
			$iter=$self->{'options_list'}->append;
			$self->{'options_list'}->set($iter,0,$o);
			$self->{'options_list'}->set($iter,1,$tmp{$o});
			delete($tmp{$o});
		}
	
		$self->{'edit_menu_win'}->set_title($title);
	
	}else{
		#do stuff here for new entry
	
	}
	
	$self->{'edit_menu_win'}->show_all;
	
}

sub on_menu_edit_win_ok_clicked{
	my ($self,$glade,$entryref)=@_;
	my %tmp;
	#warn Dumper $entryref;
	$tmp{'title'}=$self->{'title_entry'}->get_text();
	my $root_val=$self->{'root_type_menu'}->get_model()->get($self->{'root_type_menu'}->get_active_iter,0 );
	$tmp{'root'}=$self->{'root_location_entry'}->get_text() if ($root_val eq 'root');
	$tmp{'rootnoverify'}=$self->{'root_location_entry'}->get_text() if ($root_val eq 'rootnoverify');
	$tmp{'initrd'}=$self->{'initrd_entry'}->get_text() if $self->is_active($glade,'has_initrd');
	$tmp{'kernel'}=$self->{'kernel_entry'}->get_text().' '.$self->{'kernel_param_entry'}->get_text() if $self->is_active($glade,'has_kernel');	
	$tmp{'chainloader'}=$self->{'chainloader_entry'}->get_text() if $self->is_active($glade,'has_chainloader');
	if ($self->is_active($glade,'has_normal_color')){
		$tmp{'color'}=$self->{'color_fgcolor'}->get_text().'/'.$self->{'color_bgcolor'}->get_text();
		if ($self->is_active($glade,'has_hilite_color')){
			$tmp{'color'}.=' '.$self->{'color_hilite_fgcolor'}->get_text().'/'.$self->{'color_hilite_bgcolor'}->get_text();
		}
	}
	my @options;
	my $iter=$self->{'options_list'}->get_iter_first || undef;
	if ($iter){
		push @options, $self->{'options_list'}->get_value($iter,0);
		$iter=$self->{'options_list'}->iter_next($iter) || undef;
		while (defined($iter)){
			push @options, $self->{'options_list'}->get_value($iter,0);
			$iter=$self->{'options_list'}->iter_next($iter) || undef;
		}
		$tmp{'options'}=\@options;
	}
	
#	warn Dumper \%tmp;
#	$entryref=\%tmp;
	my $has_entry=0;
	if ($entryref->{'title'}){
		foreach $e (@{$self->{'entries'}}){
			if ($e->{'title'} eq $entryref->{'title'}){
				$e=\%tmp;
				$has_entry=1;
			}
		}
	}
		
	unless ($has_entry){
		push @{$self->{'entries'}}, \%tmp;
	}
	
	$self->update_all_menus unless ($entryref->{'title'} eq $tmp{'title'});
	$self->{'edit_menu_win'}->destroy;
}

#returns true if glade name passed is active
sub is_active{
	my ($self,$glade,$name)=@_;
	
	return $glade->get_widget($name)->get_active;
}

#sets up a ListStore to display devices listed from `fdisk -l`
sub setDevicesModel{
	my ($self,$model)=@_;

	my $fdisk=`/sbin/fdisk -l`;
	my @fdlist=split "\n", $fdisk;

	$model->clear;
	
	my $iter=$model->get_iter_first();
	foreach $l (@fdlist){
		if ($l=~/^\s*(\/dev\/[\S]*)\s/){
			$iter=$model->append();
			$model->set($iter,0=>$1);
		}
	}	
}

#finds the index where the passed element is located
sub findIndex{
	my ($self,$aref,$element)=@_;
	for(my $i=0;$i<@$aref;$i++){
		return $i if ( $aref->[$i] eq $element );
	}
	return -1;
}

#given a '/dev/*' device name, dev2grub() returns the corresponding grub-style device string
sub dev2grub{
	my ($self,$val)=@_;
	
	my @letters=('a','b','c','d');	
	
	$val=~/(\/dev\/)(\D*)(\d*)/;
	my ($base,$drive,$part)=($1,$2,$3);
	my $drive2=substr($drive,-1,1,'');
	my $retval="($drive".findIndex(undef,\@letters,$drive2);
	$retval.=",".($part-1) if $part;
	$retval.=")";
	
	return $retval;
}

sub on_edit_menu_options_row_activated{
	my ($self,$model,$target_entry)=@_;
	
	
}


###########   / Edit Menu Window   ##############



1;
