#!/usr/bin/perl -w

package cron2;

use Data::Dumper;
use Exporter;
use Gtk2;
use Glib ':constants';
use constant COLUMN_JOB   => 0;
use constant COLUMN_PRODUCT  => 1;
use constant COLUMN_EDITABLE => 2;
use constant NUM_COLUMNS     => 3;


@ISA=qw/Exporter Gtk2::Widget/;



sub new{
	#shift;
	#warn Dumper @_;
	shift;
	my $parent=shift;

	my $self={};
	$self=new Gtk2::VBox;
	my @crondirs=('/etc/cron.d');
	$self->{directories}=\@crondirs;
	$self->{cronfile}='/etc/crontab';

	################################
	#set up list
	$self->{jobslist}=Gtk2::TreeStore->new ('Glib::String',
											'Glib::String',
											'Glib::String');
	$self->{jobsview}=Gtk2::TreeView->new( $self->{jobslist} );
	$self->{jobsview}->set_rules_hint (TRUE);
	$self->{jobsview}->get_selection->set_mode ('single');
	$self->{jobsview}->get_selection->signal_connect(
							changed=>sub{fill_fields($self,@_);});

	$self->{row_selected}=''; #placeholder for curent selected row

	add_columns($self);
	listModel($self,'/etc/crontab'); #display contents

	################################
	#set up controls
	my $controlview=new Gtk2::Table(7,2,1);

	$self->{filename}=new Gtk2::Entry;
	$controlview->attach_defaults(new Gtk2::Label('File:'),0,1,0,1);
	$controlview->attach_defaults($self->{filename},1,2,0,1);

	$controlview->attach_defaults(new Gtk2::HSeparator,0,2,1,2);

	$self->{date}=new Gtk2::Entry;
	$controlview->attach_defaults(new Gtk2::Label('Date:'),0,1,2,3);
	$controlview->attach_defaults($self->{date},1,2,2,3);

	$self->{time}=new Gtk2::Entry;
	$controlview->attach_defaults(new Gtk2::Label('Time:'),0,1,3,4);
	$controlview->attach_defaults($self->{time},1,2,3,4);

	my $updateBtn=Gtk2::Button->new_from_stock('gtk-apply');
	#$updateBtn->set_size_request(-1,20);
	$controlview->attach_defaults($updateBtn,1,2,4,5);
	$updateBtn->signal_connect(clicked=>sub{$self->updateEntry()});

	my $deleteBtn=Gtk2::Button->new_from_stock('gtk-delete');
	#$deleteBtn->set_size_request(-1,20);
	$controlview->attach_defaults($deleteBtn,0,1,4,5);
	$deleteBtn->signal_connect(clicked=>sub{deleteEntry($self)});

	my $addBtn=Gtk2::Button->new_from_stock('gtk-add');
	#$deleteBtn->set_size_request(-1,20);
	$controlview->attach_defaults($addBtn,1,2,5,6);
	$addBtn->signal_connect(clicked=>sub{addEntry($self)});

	$controlview->attach_defaults(new Gtk2::HSeparator,0,2,6,7);


	$self->{filestat}=new Gtk2::Label('File Contents:');
	$controlview->attach_defaults($self->{filestat},0,2,7,8);
	$self->{filedisplay}=new Gtk2::TextView;
	$self->{filedisplay}->set_wrap_mode('word');
	$self->{filedisplay}->set_size_request(180,-1);

	my $w = Gtk2::ScrolledWindow->new (undef, undef);
	$w->set_shadow_type ('etched-in');
	$w->set_policy ('automatic', 'automatic');
	$w->add ($self->{filedisplay});

	my $saveBtn=Gtk2::Button->new_from_stock('gtk-save');
	#$saveBtn->set_size_request(-1,20);
	$saveBtn->signal_connect(clicked=>sub{$self->saveFile});

	my $box=new Gtk2::VBox(0,0);
	$box->pack_start($w,1,1,0);
	$box->pack_start($saveBtn,0,0,0);

	################################
	#set up main layout, add everything
	$hpaned = Gtk2::HPaned->new;
	$hpaned->set_border_width (5);

	my $sw = Gtk2::ScrolledWindow->new (undef, undef);
	$sw->set_shadow_type ('etched-in');
	$sw->set_policy ('automatic', 'automatic');
	$sw->add ($self->{jobsview});

	my $rightview=Gtk2::VPaned->new;
	$rightview->add1 ($controlview);
	$rightview->add2 ($box);
	$rightview->set_border_width (5);


	$hpaned->add1 ($sw);
	$hpaned->add2 ($rightview);
	$hpaned->signal_connect(realize=>sub{
			#warn 'allocation == '.$self->allocation()->width;
			$hpaned->set_position(($self->allocation()->width)/2);
			});


	$self->add ($hpaned);
	$self->show_all;

	bless $self;
	return $self;
}

sub fill_fields{
	#warn Dumper @_;
	my ($self,$selection)=@_;
	my $treeview = $selection->get_tree_view;
	my ($model, $iter) = $selection->get_selected;
	if ($iter && $model){
		$self->{filename}->set_text($model->get($iter,0));
		$self->{date}->set_text($model->get($iter,1));
		$self->{time}->set_text($model->get($iter,2));
		$self->{filedisplay}->get_buffer()->set_text('');

		if (open(F, $model->get($iter,0))){
			$self->{filestat}->set_text($model->get($iter,0));
			foreach (<F>){
				$self->{filedisplay}->get_buffer()->insert(
											$self->{filedisplay}->get_buffer()->get_end_iter
											,$_);
			}
			close F;
		}else{
			$parent=$model->iter_parent($iter);
			if ($parent && $model->get($parent,0)=~/run\-parts(\s*)(.*)/){
				my $dir= $2;
				if (open(F,$dir.'/'.$model->get($iter,0))){
				$self->{filestat}->set_text($dir.'/'.$model->get($iter,0));
					foreach (<F>){
						$self->{filedisplay}->get_buffer()->insert(
											$self->{filedisplay}->get_buffer()->get_end_iter
											,$_);
					}
				}
			}
		}
		if (-l $self->{filestat}->get_text){
			$self->{filestat}->set_text(readlink($self->{filestat}->get_text));
		}
	}
}

sub enactChanges{
	my $self=shift;
	puniconf::clearErr();
	puniconf::showErr();

	my $model=$self->{jobsview}->get_model;
	$model->foreach(sub{$self->doChanges(@_)});

}

sub doChanges{
	my ($self,$model,$path,$iter)=@_;
	#$model->get($iter,0);

	#warn Dumper @_;
	if ( $model && $model->iter_parent($iter) ){
		my $parent=$model->iter_parent($iter);
		if ($model->get($parent,0)=~/run\-parts(\s*)(.*)/){
			my $dir= $2;
			#puniconf::err($model->get($parent,0).":".$model->get($iter,0)."\n");
			my (@children,@files,%compare);
			getChildren($model,$parent,\@children);
			getDir($dir,\@files);
			compare(\@files,\@children,\%compare);
			foreach (keys(%compare)){
				if ($compare{$_}==0 && $_ ne '.' && $_ ne '..'){
					puniconf::err($_." == ".$compare{$_}."\n") ;
				}
			}
		}
	}else{
		puniconf::err($model->get($iter,0).":\n");
	}

	return 0;#recurse until end
}

#fills list with children of parent
sub getChildren{
	my ($model,$parent,$aref)=@_;
	@$aref=('');shift @$aref;
	for ($i=0;$i<$model->iter_n_children($parent);$i++){
		push @$aref,$model->get($model->iter_nth_child($parent,$i),0)
	}

}

#fills list with contents of dir
sub getDir{
	#warn Dumper @_;
	my ($dir,$aref)=@_;
	@$aref=('');shift @$aref;
	opendir(DIR, $dir) || return;

	foreach $f (readdir DIR){
		push @$aref,$f;
	}
	closedir DIR;
	return 0;
}

#helper method. compares 2 arrays, and fills hash with differences.
sub compare{
	my ($ref1,$ref2,$hash)=@_;

	foreach $i (@$ref1){
		my $isIn=0;
		foreach $j (@$ref2){
			if ($i eq $j){
				$isIn=1;
			}
		}
		$$hash{"$i"}=$isIn;
	}

}

sub saveFile{
	my $self=shift;
	#my $curdir=`pwd`;
	my $file=$self->{filestat}->get_text;
	my $buffer=$self->{filedisplay}->get_buffer();

	$file=~s#^../#/etc/#;

	puniconf::showStatus($file);
	if ((-x $file) && open(F,'>'.$file)){
		print F $buffer->get_text($buffer->get_start_iter,$buffer->get_end_iter,1);
		close F;
		puniconf::showStatus('Successfully updated'.$file);
	}else{puniconf::showStatus('Could not update'.$file);}

	#chdir $curdir;
}

sub deleteEntry{
	#warn Dumper @_;
	my $self=shift;
	my $selection=$self->{jobsview}->get_selection;
	my ($model, $iter) = $selection->get_selected;
	my $response='yes';
	if ($iter && $model){
		if($model->iter_has_child($iter)){
			my $d=Gtk2::MessageDialog->new(
					puniconf::getWindow(),
					'GTK_DIALOG_MODAL',
					'GTK_MESSAGE_QUESTION',
					'GTK_BUTTONS_YES_NO',
					$model->get($iter,0)." has children.\n".
						"Are you sure you want to delete all of them?"
					);
			$response=$d->run;
			$d->destroy();
		}

		if ($response eq 'yes'){
			if ($model->remove($iter)){
				puniconf::showStatus("Deleted entry");
			}else{
				puniconf::showStatus("Entry not changed");
			}
		}
	}else{
		puniconf::showStatus("Nothing selected!");
	}
}

sub updateEntry{
	#warn Dumper @_;
	my $self=shift;
	my $selection=$self->{jobsview}->get_selection;
	my ($model, $iter) = $selection->get_selected;
	$self->{jobslist}->set ($iter,
					0=>$self->{filename}->get_text(),
					1=>$self->{date}->get_text(),
					2=>$self->{time}->get_text()
					);
	puniconf::showStatus("Updated entry...");
}

sub migrate{
	my $self=shift;

	my @finfo=stat '/etc/cron.d';
	$mode=$finfo[2];
	my $rundir='/etc/cron.run';

	unless (-x $rundir){
		if(mkdir $rundir,$mode){
			warn $rundir.' successfully created with mode = '.$mode;
		}else{warn $rundir.'not created! error:\n'.$!;}
	}

	foreach $dir (@{$self->{directories}}){
		if (opendir DIR, $dir){
			foreach $file (readdir DIR){
				unless (-l $dir.'/'.$file){
					rename $dir.'/'.$file, $rundir.'/'.$file;
					symlink $rundir.'/'.$file, $dir.'/'.$file;
				}
			}
			closedir DIR;
		}
	}

}


sub add_columns {
	#warn Dumper @_;
	my $self=shift;
  # number column
  my $renderer = Gtk2::CellRendererText->new;
  $self->{jobsview}->append_column (Gtk2::TreeViewColumn->new_with_attributes ("Job", $renderer,
					    text => 0));

  # product column
  $renderer = Gtk2::CellRendererText->new;
  $self->{jobsview}->append_column (Gtk2::TreeViewColumn->new_with_attributes ("Date", $renderer,
					    text => 1));

  $renderer = Gtk2::CellRendererText->new;
  $self->{jobsview}->append_column (Gtk2::TreeViewColumn->new_with_attributes ("Time", $renderer,
					    text => 2));
}

sub listModel{
	#warn Dumper @_;
	my $self=shift;
	my $file=shift;
	my @jobs=();
	$self->{jobslist}->clear();

	if (open(CRON,$file)){
		my $iter;
			my %job;
			#=(name=>'',minute=>'',hour=>'',day=>'',month=>'',user=>'',weekday=>'');
		foreach $l (<CRON>){
			my @line=split ' ',$l;
			if (@line>=5){
				my $fileIter;
				$job{'minute'}=shift @line;
				$job{'hour'}=shift @line;
				$job{'day'}=shift @line;
				$job{'month'}=shift @line;
				$job{'weekday'}=shift @line;
				$job{'user'}=shift @line;
				$job{'name'}=join(' ',@line);
				#warn $job{'name'};
				my $dirIter=add_row($self,$fileIter,%job);
				if ($job{'name'}=~/run\-parts(\s*)(.*)/){
					my $dir= $2;
					if (opendir(DIR, $dir)){
						#warn 'opened';
						my @tmp=@{$self->{directories}};
						push @tmp,$dir;
						$self->{directories}=\@tmp;
						foreach $m ( readdir( DIR ) ){
							$job{'name'}=$m;
							add_row($self,$dirIter,%job);
						}
						closedir DIR;
					}
				}
			}
		}
		close CRON;
 	}else{
 		push @jobs, {name=>'cannot open file!'};
 	}

}
sub add_row{
	#warn Dumper @_;
	my $self=shift;
	my $parent=shift;
	my %job=@_;
	unless ($job{'name'} eq '.' || $job{'name'} eq '..'){
		my $iter = $self->{jobslist}->append($parent);
		$self->{jobslist}->set ($iter,
						0=>$job{'name'},
						1=>join('-', ($job{'month'}, $job{'day'}, $job{'weekday'})),
						2=> join(':', ($job{'hour'}, $job{'minute'}))
						);
	return $iter;
	}
}

##################
#
#All methods below pertain to UI and events
#
##################
sub getHelp{
	#warn Dumper @_;
	my $self=shift;
	my $buffer=shift;
	my $iter = $buffer->get_end_iter;

	$buffer->insert_with_tags_by_name ($iter, "CronConfig\n", "helpheading");
	$buffer->insert_with_tags_by_name ($iter, "About\n", "helptopic");
	$buffer->insert_with_tags_by_name ($iter, "CronConfig is a configuration system for Linux crontabs.", "helptopicbody");


}

sub getLabel{
	my $box=Gtk2::HBox->new(0,0);
	my $icon=Gtk2::Image->new_from_stock('gtk-preferences','GTK_ICON_SIZE_MENU');
	my $text=new Gtk2::Label("CronConfig");
	my $closebtn=new
	$box->pack_start($icon,0,0,0);
	$box->pack_start($text,1,1,0);
	$box->show_all;
	return $box;
}

sub showHelp{
	puniconf::showStatus("This is where the help goes.");
}

#open the crontab file
sub Open{
	#warn Dumper @_;
	my $self=shift;
	#listModel($self->{jobsview},'another one...');
	my $file='';
	my $filesel=Gtk2::FileSelection->new('Select a cron file...');
	$filesel->set_filename('/etc/crontab');
	$filesel->show();
	$filesel->cancel_button->signal_connect(clicked=>sub{$filesel->destroy()});
	$filesel->ok_button->signal_connect(
						clicked=>sub{
								$self->listModel($filesel->get_filename());
								$filesel->destroy();
								});
	#warn Dumper $file;

}
sub Save{
	puniconf::showStatus("cron Save...");
}
sub Focus{
	puniconf::showStatus("cron has the focus...");
}
sub setMenu{
	my ($self)=@_;
	my $plugmnu = Gtk2::Menu->new;
		my $aboutItem=Gtk2::MenuItem->new_with_mnemonic("cron TestPlug2...");
		$aboutItem->signal_connect (activate => sub{doAbout()});
		$plugmnu->append($aboutItem);
		my $helpItem=Gtk2::MenuItem->new_with_mnemonic("cron _Help...");
		$helpItem->signal_connect (activate => sub{doHelp()});
		$plugmnu->append($helpItem);

	puniconf::addMenu("cron",$plugmnu);
}

sub doAbout{
	puniconf::showStatus("cron About...");
}
sub doHelp{
	puniconf::showStatus("cron Help...");
}

sub getToolbarItems{
	my ($self)=@_;

	my $migrateTTip='Migrate to usable operations..';
	my $migratePTTip='migrate';
	my $migrateBtn=Gtk2::ToolButton->new (
					Gtk2::Image->new_from_stock(
						'gtk-convert',
						'GTK_ICON_SIZE_LARGE_TOOLBAR'),
					'Migrate...');
		$migrateBtn->signal_connect (clicked => sub{$self->migrate()});
	my $enactTTip='Enact changes...';
	my $enactPTTip='enact';
	my $enactBtn=Gtk2::ToolButton->new (
					Gtk2::Image->new_from_stock(
						'gtk-apply',
						'GTK_ICON_SIZE_LARGE_TOOLBAR'),
					'Enact Changes...');
		$enactBtn->signal_connect (clicked => sub{$self->enactChanges()});



	puniconf::addTbItem(
				$migrateBtn,$migrateTTip,$migratePTTip,
				$enactBtn,$enactTTip,$enactPTTip);

}
sub ToolBtnClick{
	puniconf::showStatus("cron Toolbar Button was clicked...");
}


sub getIcon{
	my $icon=puniconf::getWindow->render_icon("gtk-preferences", "menu");
	#warn Dumper $icon;
	return $icon;
}







1;
