#!/usr/bin/perl -w
#
# puniconf
#
# Copyright (C) July, 2003, Skaman Sam <SkamanSam@punks.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


# ##################################################################################
# This space is reserved for general notes about the program.
########
# NOTE: This program uses the following Perl modules. You can get them via
#		CPAN or from http://search.cpan.org
#		Gtk2, Gtk2::Pango, Glib		http://gtk2-perl.sf.net
#				These are the main UI packages.
#		Data::Dumper 				(perl core module)
#				To aid in knowing what parameters are passed to subroutines
#		Devel::Size					http://www.sidhe.org/~dan/blog/
#				To help in downsizing memory footprint
#		Religion					(no known homepage)
#				To redirect errors to dialog boxes. I tweaked this module a bit,
#				so I have included it here.
#
########
# NOTE: This is a re-write from scratch of fbconfig. It uses an OOP approach.
#		My last version was lost due to a corrupt backup disk, and took me
#		six months, from designing, to implement. I was still learning Perl
#		and Gtk+ programming, and went from Gtk+ 1.2 to Gtk+ 2.0 half way through.
#		This re-write took me three days. WOW! I have come far, I believe, but
#		still am very far from where I would like. If you would like to see
#		a plugin for puniconf, e-mail me at SkamanSam@punks.net, and I will
#		try my best to write one for you. Thank you for your support.
########
# NOTE: The best way to set up the plugins is by changing the user to root
#		if the plugins writes to root-owned files. Use the command
#		'chown root: plugins/myplugin.pm'. and change the permissions so
#		only root can read: 'chmod 400 plugins/myplugin.pm'. puniconf will
#		use only plugins it can open. FIX: if i ever move to the GtkPlug
#		interface, you will only need to run those plugins you want.
########
# NOTE: The memory requirements of puniconf seem to be due to Perl-Gtk2 libs.
#		The demo app that comes with it is just under 8Mb, with no demos running.
#		puniconf takes up about 12Mb with no plugins. As a general rule, allow 1Mb
#		for every plugin running. I am trying to work on lowering the footprint.
#
# ##################################################################################


package puniconf;

use Gtk2 -init;
use Glib ':constants';
use Data::Dumper;
use GUI::Religion;
use Devel::Size;
use Gtk2::GladeXML; 


my $glade=Gtk2::GladeXML->new("glade/puniconf.glade");

#check to see if program is setup correctly.
# if not, set it up
#checkDirs();
my $config=readPrefs($ENV{'HOME'}."/.puniconf/prefs/puniprefs");


#do splash screen stuff
#btu how...?





#import UI elements into namespace.
#NOTE: if anyone can find a way to do this better, let me know,
#		but I think this is the best way.
foreach $l (<GUI/*.pm>){
	my @path=split '/',$l;
	my $mname=@path[@path-1];
	$mname=~s/.pm//;
	#print 'use GUI::'.$mname.';';
	eval('use GUI::'.$mname.';');
	warn $@ if $@;

}


#initialize interface
my $fbuffer=new GUI::TextBuffer;
my $win=new GUI::MainWindow($glade);
my $tools=new GUI::ToolBar($glade);
my $menu=new GUI::Menu($glade);
my $notes=new GUI::NoteBook($glade,$win,$menu);
my $status=new GUI::StatusBar($glade);
my $output=new GUI::OutputWindow($glade);
my $help=new GUI::HelpWindow($glade);
my $prefs=new GUI::PrefsWindow($glade);
my $tooltips=new Gtk2::Tooltips();

#import and initialize all plugins
$win->show();

#NOTE: if someone knows a better way...
foreach $l (<plugins/*.pm>){     #((<$plugsDir/*.pm>,<$localplugsDir/*.pm>)){
	if (open(P,$l)){
		my @path=split '/',$l;
		my $mname=@path[@path-1];
		$mname=~s/.pm//;
		eval('use plugins::'.$mname.';'.
			'$notes->append_page('.
				$mname.'->new($win),'.
				$mname.'->getLabel());'.
			$mname.'->setMenu($menu);') if (isInArray(($config->{'plugins'}),$mname));
		warn $@ if $@;
		close P;
	}
}
doTabFocus(undef,undef,0);

#####################
# GtkPlug Testing area.


######################
#usd to handle die and warn events.
#my $eh=new GUI::ErrorHandler($glade);
$Warn::Handler = new WarnHandler(\&GUI::ErrorHandler::doError);
$Die::Handler = new DieHandler(\&GUI::ErrorHandler::dieError);



#start the Gtk2 engine
Gtk2->main;
exit 0;


#############################
#
# Plugin Access functions
#	These functions are called by plugins.
#
# ###########################

#sub getPluginsPath{return $plugsDir;}
#sub getGladePath{return $gladeDir;}
#sub getLocalGladePath{return $localgladeDir;}
#sub getLocalPluginsPath{return $localplugsDir;}

#getter methods for UI elements
sub getWindow{return $win;}
#sub getLayout{return $layout;}
sub getToolBar{return $tools;}
sub getMenu{return $menu;}
sub getNoteBook{return $notes;}
sub getErrWindow{return $output;}
sub getHelpWindow{return $help;}
sub getStatusBar{return $status;}
sub getBuffer{return new GUI::TextBuffer;}
sub getPrefs{return new $prefs;}
sub getTips{return $tooltips;}

sub addTip{
	my ($widget,$tip,$ptip)=@_;
	$tooltips->set_tip(@_);
}
sub showTips{$tooltips->enable;}
sub hideTips{$tooltips->disable;}

sub showPrefs {
	$prefs->show_all();
}

sub addMenu{
	$menu->addPlugMenu(@_);
}

sub err{
	$output->add(@_);
}

sub clearErr{
	$output->clear();
}

sub showErr{
	$output->show_all();
}

sub stockTab{
	my ($label,$iconStr,$ttip)=@_;

	my $box=Gtk2::HBox->new(0,0);
	my $icon=Gtk2::Image->new_from_stock($iconStr,'menu');
	my $text=new Gtk2::Label($label);
	$box->pack_start($icon,0,0,0);
	$box->pack_start($text,0,0,0);
	$box->show_all;
		addTip($text,$ttip,undef);
		showTips;
	return $box;

}
sub getFileSave{
	#Gtk2::FileChooserActions: open, save, create-folder, select-folder
	my ($title)=@_;
	my $dlg=new Gtk2::FileChooserDialog($title, 
										getWindow(), 
										'save',
										'gtk-ok'=>'ok','gtk-cancel'=>'cancel');
	my $response=$dlg->run;
	my $name='';
	if ($response eq 'ok'){
		$name=$dlg->get_filename;
		#warn "You have selected \n$name.";
	}
	$dlg->destroy;
	return $name;	
}

sub getFileOpen{
	#Gtk2::FileChooserActions: open, save, create-folder, select-folder
	my ($title)=@_;
	my $dlg=new Gtk2::FileChooserDialog($title, 
										getWindow(), 
										'open',
										'gtk-ok'=>'ok','gtk-cancel'=>'cancel');
	my $response=$dlg->run;
	my $name='';
	if ($response eq 'ok'){
		$name=$dlg->get_filename;
		#warn "You have selected \n$name.";
	}
	$dlg->destroy;
	return $name;	

}
sub getFolderOpen{
	#Gtk2::FileChooserActions: open, save, create-folder, select-folder
	my ($title)=@_;
	my $dlg=new Gtk2::FileChooserDialog($title, 
										getWindow(), 
										'select-folder',
										'gtk-ok'=>'ok','gtk-cancel'=>'cancel');
	my $response=$dlg->run;
	my $name='';
	if ($response eq 'ok'){
		$name=$dlg->get_filename;
		#warn "You have selected \n$name.";
	}
	$dlg->destroy;
	return $name;	

}

sub quit{
	$win->destroy;
	#warn "Quitting...";
	Gtk2->main_quit;
}


sub showStatus {
  my ($txt) = @_;

  $status->pop (0); # clear any previous message, underflow is allowed

  $status->push (0, $txt);
}

sub addTbItem{$tools->addButtons(@_);}

sub showAbout{
		#call doAbout() in active plugin
		$notes->get_nth_page($notes->get_current_page)->doAbout();
}

#############################
#
# Internal or Event Methods
#	These methods are called when certain events occur.
#
# ###########################

#sets up all user and global directories
# this method is basically an installation routine
#sub checkDirs{
#	
#	my $t=makeDir( $shareDir );
#	$t=makeDir( $prefsDir );
#	copyDir( "GUI/",$GUIDir ) if ( makeDir( $GUIDir ) == 1 );
#	copyDir( "plugins/",$plugsDir ) if ( makeDir( $plugsDir ) == 1 );
#	copyDir( "glade/",$gladeDir ) if ( makeDir( $gladeDir ) == 1 );
#	
#	$t=makeDir( $localDir );
#	$t=makeDir( $localprefsDir );
#	$t=makeDir( $localGUIDir );
#	$t=makeDir( $localplugsDir );
#	$t=makeDir( $localgladeDir );
#	
#	unless (-e $localprefsFile){
#		if (open (PREFS,">localprefs")){
#			foreach $l ((<$plugsDir/*.pm>,<$localplugsDir/*.pm>)){
#				if (open(DIRS,$l)){
#					my @path=split '/',$l;
#					my $mname=@path[@path-1];
#					$mname=~s/.pm//;
#					print PREFS "plugin=$mname\n";
#					close DIRS;
#				}
#			}
#			
#			close PREFS;
#		}
#	}
#	
#}

#copies one dir to another
sub copyDir{
	my ($from,$to)=@_;
	print "Copying all files in $from to $to ...\n";
	#foreach $f (<$from/*>){
	#	print "Copying $f to $to/$f...\n";
	#}
	print `cp $from/* $to/`;
	 
}

#checks to see if DIR exists, and creates it
#return 0 on failure, 1 on success, and -1 if nothing done
sub makeDir{
	my ($dir)=@_;
	unless (-e $dir){
		print "$dir doesn't exist...";
		mkdir $dir;
		if (-e $dir){
			print "but was created!\n";
		}else{ 
			print "and could NOT be created!\n"; 
			return 0;
		}
		return 1;
	}
	return -1;
}

#get prefs as a hash reference
sub readPrefs{
	my ($file)=@_;
	my %h;
	my @plugins;
	unless (-e $file){
		open PREFS,">$file";
		foreach $l (<plugins/*.pm>){
			if (open(DIRS,$l)){
				my @path=split '/',$l;
				my $mname=@path[@path-1];
				$mname=~s/.pm//;
				print PREFS "plugin=$mname\n";
				close DIRS;
			}
		}		
		close PREFS;
	}
	if (open(PREFS,$file)){
		
		foreach $l (<PREFS>){
			chomp $l;
			if ( $l=~/^\s*plugin\s*=\s*(\S*)/ ){
				push ( @plugins, $1 );
			}
		}
		$h{'plugins'}=\@plugins;
		close PREFS;
	}
	return \%h;
}

#finds out if a string is a member of an array
sub isInArray{
	my ($aref,$element)=@_;
	foreach $e (@$aref){
		return 1 if $e eq $element;
	}
	return 0;
}

# the user has selected Open... either in the menu or the toolbar
sub doOpenItem{
	showStatus("Open Item Selected!");
	my $curPage=$notes->get_nth_page($notes->get_current_page());
	$curPage->Open();
}

# the user has selected Save... either in the menu or the toolbar
sub doSaveItem{
	showStatus("Save Item Selected!");
	my $curPage=$notes->get_nth_page($notes->get_current_page());
	$curPage->Save();
}

# the user has clicked on a tab, making it (the plugin) active
sub doTabFocus{
	#warn Dumper @_;
	my ($notebook,$page,$pageNum)=@_;
	#warn "In doTabFocus()...\n";
	if ($notes->get_nth_page($pageNum)){
		my $curPage=$notes->get_nth_page($pageNum);

		#clear toolbar and add active plugin's items
		$tools->clear();
		$curPage->getToolbarItems();

		#tell plugin to do something when tab gets the focus
		$curPage->Focus();

		#set the icon now, and for all new windows
		$win->set_icon($curPage->getIcon());
		$win->set_default_icon($curPage->getIcon());
	}
}

1;
